#include <Arduino.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define s0 8 // Module pins wiring
#define s1 9
#define s2 10
#define s3 11
#define out 12

#define BUZZER_PIN 3 // Define the pin for the passive buzzer

// Initialize the library by specifying the LCD address (usually 0x27 or 0x3F) and size (16x2)
LiquidCrystal_I2C lcd(0x27, 16, 2);

int Red = 0, Blue = 0, Green = 0; // RGB values

void setup()
{
	pinMode(s0, OUTPUT); // pin modes
	pinMode(s1, OUTPUT);
	pinMode(s2, OUTPUT);
	pinMode(s3, OUTPUT);
	pinMode(out, INPUT);
	pinMode(BUZZER_PIN, OUTPUT); // Set the buzzer pin as output

	Serial.begin(9600);		   // Initialize the serial monitor baud rate
	lcd.begin(16, 2);		   // Initialize the LCD with 16 columns and 2 rows
	lcd.backlight();		   // Turn on the backlight
	lcd.print("Color Sensor"); // Print a message to the LCD

	digitalWrite(s0, HIGH); // Putting S0/S1 on HIGH/HIGH levels means the output frequency scaling is at 100% (recommended)
	digitalWrite(s1, HIGH); // LOW/LOW is off, HIGH/LOW is 20%, and LOW/HIGH is  2%
}

void GetColors()
{
	digitalWrite(s2, LOW); // S2/S3 levels define which set of photodiodes we are using LOW/LOW is for RED, LOW/HIGH is for Blue, and HIGH/HIGH is for Green
	digitalWrite(s3, LOW);
	Red = pulseIn(out, digitalRead(out) == HIGH ? LOW : HIGH); // Wait until "out" goes LOW, start measuring the duration, and stop when "out" is HIGH again
	delay(20);
	digitalWrite(s3, HIGH); // Select the other color (set of photodiodes) and measure the other colors' values using the same technique
	Blue = pulseIn(out, digitalRead(out) == HIGH ? LOW : HIGH);
	delay(20);
	digitalWrite(s2, HIGH);
	Green = pulseIn(out, digitalRead(out) == HIGH ? LOW : HIGH);
	delay(20);
}

void playTone(int frequency, int duration)
{
	tone(BUZZER_PIN, frequency, duration); // Play the tone on the passive buzzer
	delay(duration);					   // Wait for the tone to finish
	noTone(BUZZER_PIN);					   // Stop the tone
}

void loop()
{
	GetColors(); // Execute the GetColors function to get the value of each RGB color
				 // Depending on the RGB values given by the sensor, we can define the color and display it on the monitor

	lcd.clear(); // Clear the LCD display

	if (Red <= 15 && Green <= 15 && Blue <= 15) // If the values are low, it's likely the white color (all the colors are present)
	{
		Serial.println("White");
		lcd.setCursor(0, 0);
		lcd.print("Color: White");
		playTone(1000, 500); // Play a tone for white
	}
	else if (Red < Blue && Red <= Green && Red < 23) // If Red value is the lowest one and smaller than 23, it's likely Red
	{
		Serial.println("Red");
		lcd.setCursor(0, 0);
		lcd.print("Color: Red");
		playTone(500, 500); // Play a different tone for red
	}
	else if (Blue < Green && Blue < Red && Blue < 20) // Same thing for Blue
	{
		Serial.println("Blue");
		lcd.setCursor(0, 0);
		lcd.print("Color: Blue");
		playTone(1500, 500); // Play a different tone for blue
	}
	else if (Green < Red && Green - Blue <= 8) // Green was a little tricky; you can do it using the same method as above (the lowest), but here I used a reflective object
	{
		Serial.println("Green"); // Which means the blue value is very low too, so I decided to check the difference between green and blue and see if it's acceptable
		lcd.setCursor(0, 0);
		lcd.print("Color: Green");
		playTone(1000, 500); // Play a different tone for green
	}
	else if (Red > 10 && Red < 21 && Green > 10 && Green < 21 && Blue > 10 && Blue < 21) // Conditions for Yellow
	{
		Serial.println("Yellow");
		lcd.setCursor(0, 0);
		lcd.print("Color: Yellow");
		playTone(2000, 500); // Play a different tone for yellow
	}
	else if (Red < Green && Red < Blue && abs(Green - Blue) <= 2) // Conditions for Orange
	{
		Serial.println("Orange");
		lcd.setCursor(0, 0);
		lcd.print("Color: Orange");
		playTone(2200, 500); // Play a different tone for orange
	}
	else if (Red > 50 && Green < 50 && Blue > 50) // Conditions for Purple (lowered thresholds)
	{
		Serial.println("Purple");
		lcd.setCursor(0, 0);
		lcd.print("Color: Purple");
		playTone(2400, 500); // Play a different tone for purple
	}
	else if (Red > 50 && Green > 50 && Blue > 50) // Conditions for Black
	{
		Serial.println("Black");
		lcd.setCursor(0, 0);
		lcd.print("Color: Black");
		playTone(2600, 500); // Play a different tone for black
	}
	else
	{
		Serial.println("Unknown"); // If the color is not recognized, you can add as many as you want
		lcd.setCursor(0, 0);
		lcd.print("Color: Unknown");
		playTone(2500, 500); // Play a different tone for unknown
	}

	lcd.setCursor(0, 1); // Move to the second line
	lcd.print("R:");
	lcd.print(Red);
	lcd.print(" G:");
	lcd.print(Green);
	lcd.print(" B:");
	lcd.print(Blue);

	delay(2000); // 2s delay; you can modify if you want
}
